/* ESP32 +TFT + Muses72323 Volume Controller
*********************

Author Geoff Webster

Initial version 1.0 July 2024
- SetVolume routine to display atten/gain (-111.75dB min to 0dB)

*/

#include <Arduino.h>
#include <Preferences.h>
#include <SPI.h>
#include <TFT_eSPI.h> // Hardware-specific library
#include <RC5.h>
#include <Muses72323.h> // Hardware-specific library
#include <ESP32RotaryEncoder.h>
#include <MCP23S08.h> // Hardware-specific library
#include <LittleFS.h>
#include <FS.h>
#include "Free_Fonts.h" // Include the Free fonts header file

// Current software
#define softTitle1 "ESP32/TFT"
#define softTitle2 "Remote Command Test"
// version number
#define VERSION_NUM "1.0"

/******* MACHINE STATES *******/
#define STATE_BALANCE 1 // when user adjusts balance
#define STATE_RUN 0     // normal run state
#define STATE_IO 1      // when user selects input/output
#define STATE_OFF 4     // when power down
#define ON LOW
#define OFF HIGH
#define STANDBY 0 // Standby
#define ACTIVE 1  // Active

// Preference modes
#define RW_MODE false
#define RO_MODE true

#define TIME_EXITSELECT 5 //** Time in seconds to exit I/O select mode when no activity

Preferences preferences;

// 23S08 Construct
MCP23S08 MCP(10); //  HW SPI address 0x00, CS GPIO10

TFT_eSPI tft = TFT_eSPI(); // Invoke custom library

// define IR input
unsigned int IR_PIN = 27;
// RC5 construct
RC5 rc5(IR_PIN);

// define preAmp control pins
const int s_select_72323 = 16;
//  The address wired into the muses chip (usually 0).
static const byte MUSES_ADDRESS = 0;

// preAmp construct
static Muses72323 Muses(MUSES_ADDRESS, s_select_72323); // muses chip address (usually 0), slave select pin0;

// define encoder pins
const uint8_t DI_ENCODER_A = 33;
const uint8_t DI_ENCODER_B = 32;
const int8_t DI_ENCODER_SW = 12;

// Rotary construct
RotaryEncoder rotaryEncoder(DI_ENCODER_A, DI_ENCODER_B, DI_ENCODER_SW);

/******* TIMING *******/
unsigned long milOnButton;  // Stores last time for switch press
unsigned long milOnAction;  // Stores last time of user input
unsigned long milOnFadeIn;  // LCD fade timing
unsigned long milOnFadeOut; // LCD fade timing

/********* Global Variables *******************/

float atten;     // current attenuation, between 0 and -111.75
int16_t volume; // current volume, between 0 and -447
bool backlight;  // current backlight state
uint16_t counter = 0;
uint8_t source;        // current input channel
uint8_t oldsource = 1; // previous input channel
bool isMuted;          // current mute status
uint8_t state = 0;     // current machine state
uint8_t balanceState;  // current balance state
bool btnstate = 0;
bool oldbtnstate = 0;
int lastSeconds = 0;    // last seconds
int currentSeconds = 0; // current seconds

/*System addresses and codes used here match RC-5 infra-red codes for amplifiers (and CDs)*/
uint16_t oldtoggle;
u_char oldaddress;
u_char oldcommand;
u_char toggle;
u_char address;
u_char command;

// Used to know when to fire an event when the knob is turned
volatile bool turnedRightFlag = false;
volatile bool turnedLeftFlag = false;

char buffer1[20] = "";

// Global Constants
//------------------
const char *inputName[] = {"  Phono ", "   Media  ", "     CD    ", "   Tuner  "}; // Elektor i/p board
const int source_size = 6;
const int volume_size = 6;
const int source_x = 100;
const int source_y = 10;
const int volume_x = 100;
const int volume_y = 100;

// Function prototypes
void runTest(void);

void runTest()
{  unsigned char toggle;
  unsigned char address;
  unsigned char command;
  if (rc5.read(&toggle, &address, &command))
  {
    Serial.print("a:");
    Serial.print(address);
    Serial.print(" c:");
    Serial.print(command);
    Serial.print(" t:");
    Serial.println(toggle);
    sprintf(buffer1, "Address: %b  ", address);
    tft.drawString(buffer1, 160, 80, 1);
    sprintf(buffer1, "command: %b  ", command);
    tft.drawString(buffer1, 160, 120, 1);
    sprintf(buffer1, "t: %b  ", toggle);
    tft.drawString(buffer1, 160, 200, 1);
  }
}

// This section of code runs only once at start-up.
void setup()
{
  Serial.begin(115200);

  // Initialise the TFT screen
  tft.init();
  tft.setRotation(1);

  // Set text datum to middle centre
  tft.setTextDatum(MC_DATUM);
  tft.setFreeFont(FSS18);

  // Clear the screen
  tft.fillScreen(TFT_WHITE);

  // show software version briefly in display
  tft.setTextColor(TFT_BLUE, TFT_WHITE);
  tft.drawString(softTitle1, 160, 80, 1);
  tft.drawString(softTitle2, 160, 120, 1);
  tft.drawString("Test ver " VERSION_NUM, 160, 160, 1);
  delay(2000);
  tft.setFreeFont(FSS24);
  tft.fillScreen(TFT_WHITE);
}

void loop()
{
  runTest();
}
